﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectElectricLocomotive.DrawingObjects;
using ProjectElectricLocomotive.MovementStrategy;

namespace ProjectElectricLocomotive.Generics
{
    internal class LocomotiveGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>> _locomotivesStorage;

        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _locomotivesStorage.Keys.ToList();


        private readonly int _pictureWidth;

        private readonly int _pictureHeight;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public LocomotiveGenericStorage(int pictureWidth, int pictureHeight)
        {
            _locomotivesStorage = new Dictionary<string, LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_locomotivesStorage.ContainsKey(name))
            {
                _locomotivesStorage.Add(name, new LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>(_pictureWidth, _pictureHeight));
            }
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (_locomotivesStorage.ContainsKey(name))
            {
                _locomotivesStorage.Remove(name);
            }
        }


        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>?
        this[string ind]
        {
            get
            {
                if (_locomotivesStorage.ContainsKey(ind))
                {
                    return _locomotivesStorage[ind];
                }
                return null;
            }
        }

        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>> record in _locomotivesStorage)
            {
                StringBuilder records = new();
                foreach (DrawingLocomotive? elem in record.Value.GetLocomotives)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using FileStream fs = new(filename, FileMode.Create);
            byte[] info = new
            UTF8Encoding(true).GetBytes($"LocomotiveStorage{Environment.NewLine}{data}");
            fs.Write(info, 0, info.Length);
            return true;
        }
        /// <summary>
        /// Загрузка информации по автомобилям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            string bufferTextFromFile = "";
            using (FileStream fs = new(filename, FileMode.Open))
            {
                byte[] b = new byte[fs.Length];
                UTF8Encoding temp = new(true);
                while (fs.Read(b, 0, b.Length) > 0)
                {
                    bufferTextFromFile += temp.GetString(b);
                }
            }
            var strs = bufferTextFromFile.Split(new char[] { '\n', '\r' },
            StringSplitOptions.RemoveEmptyEntries);
            if (strs == null || strs.Length == 0)
            {
                return false;
            }
            if (!strs[0].StartsWith("LocomotiveStorage"))
            {
                //если нет такой записи, то это не те данные
                return false;
            }
            _locomotivesStorage.Clear();
            foreach (string data in strs)
            {
                string[] record = data.Split(_separatorForKeyValue,
                StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 2)
                {
                    continue;
                }
                LocomotiveGenericCollection<DrawingLocomotive, DrawingObjectLocomotive>
                collection = new(_pictureWidth, _pictureHeight);
                string[] set = record[1].Split(_separatorRecords,
                StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    DrawingLocomotive? loco =
                    elem?.CreateDrawingLocomotive(_separatorForObject, _pictureWidth, _pictureHeight);
                    if (loco != null)
                    {
                        if ((collection + loco) != -1) // or vice versa
                        {
                            return false;
                        }
                    }
                }
                _locomotivesStorage.Add(record[0], collection);
            }
            return true;
        }
    }
}
